// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "PokerPlayer.hpp"

/** The default blind cost for the big blind. */
#define POKER_BLIND_BIG_DEFAULT 600
/** The default blind cost for the small blind. (Defaults half big blind) */
#define POKER_BLIND_SMALL_DEFAULT (POKER_BLIND_BIG_DEFAULT/2)

/** How many cards are dealt for the flop, turn and river */
#define POKER_FLOP_CARD_COUNT 3
#define POKER_TURN_CARD_COUNT 1
#define POKER_RIVER_CARD_COUNT 1

#define PLAYER_COUNT_MAX 8
#define PLAYER_COUNT_MIN 2

namespace Dawn {
  class PokerGame : public std::enable_shared_from_this<PokerGame> {
    protected:
      std::vector<struct Card> deck;
      std::vector<struct Card> grave;
      uint8_t dealerIndex;
      uint8_t smallBlindIndex;
      uint8_t bigBlindIndex;
      int32_t smallBlind = POKER_BLIND_SMALL_DEFAULT;
      int32_t bigBlind = POKER_BLIND_BIG_DEFAULT;

      bool_t hasFlopped = false;
      bool_t hasTurned = false;
      bool_t hasRivered = false;

    public:
      std::vector<std::shared_ptr<PokerPlayer>> players;
      std::vector<struct PokerPot> pots;
      std::vector<struct Card> community;
      uint8_t betterIndex;

      /**
       * Creates and adds a new player to the game.
       */
      std::shared_ptr<PokerPlayer> addNewPlayer();

      /**
       * Returns the player that is currently the better.
       * 
       * @return The player that is currently the better.
       */
      std::shared_ptr<PokerPlayer> getCurrentBetter();

      /**
       * Starts a new game of poker.
       */
      void newGame();

      /**
       * Starts a new round of poker.
       */
      void newRound();

      /**
       * Starts a new betting round.
       */
      void newBettingRound();

      /**
       * Takes the blinds from the players.
       */
      void takeBlinds();

      /**
       * Sets the blinds for the game.
       * 
       * @param small The cost of the small blind.
       * @param big The cost of the big blind.
       */
      void setBlinds(const int32_t small, const int32_t big);

      /**
       * Returns the count of players that still need to bet this round.
       * 
       * @return The count of players that still need to bet this round.
       */
      uint8_t getRemainingBettersCount();

      /**
       * Returns the current call value for the game.
       * 
       * @return The current call value for the game.
       */
      int32_t getCurrentCallValue();

      /**
       * Returns the next better index.
       * 
       * @return The next better index.
       */
      uint8_t getNextBetterIndex();

      /**
       * Sets the dealer for the game.
       * 
       * @param dealer The index of the dealer.
       */
      void setDealer(const uint8_t dealer);

      /**
       * Sends a card to the burn pile.
       */
      void burnCard();

      /**
       * Deals a card to a player.
       * 
       * @param player The player to deal the card to.
       */
      void dealCard(PokerPlayer &player);

      /**
       * Deals a card to each player.
       * 
       * @param count The count of cards to deal.
       */
      void dealToEveryone(const uint8_t count);

      /**
       * Deals a card to the community.
       * 
       * @param count The count of cards to turn.
       */
      void turn(const uint8_t count = 0xFF);

      /**
       * Returns the count of cards that need to be turned.
       * 
       * @return The count of cards that need to be turned.
       */
      uint8_t getCountOfCardsToTurn();

      /**
       * Returns the count of players that are still in the game.
       * 
       * @return The count of players that are still in the game.
       */
      uint8_t getRemainingPlayersCount();
  };
}