// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PokerWinning.hpp"
#include "PokerPlayer.hpp"

using namespace Dawn;

float_t PokerWinning::getWinningTypeConfidence(enum PokerWinningType type) {
  switch(type) {
    case PokerWinningType::RoyalFlush:
      return POKER_WINNING_CONFIDENCE_ROYAL_FLUSH;
    case PokerWinningType::StraightFlush:
      return POKER_WINNING_CONFIDENCE_STRAIGHT_FLUSH;
    case PokerWinningType::FourOfAKind:
      return POKER_WINNING_CONFIDENCE_FOUR_OF_A_KIND;
    case PokerWinningType::FullHouse:
      return POKER_WINNING_CONFIDENCE_FULL_HOUSE;
    case PokerWinningType::Flush:
      return POKER_WINNING_CONFIDENCE_FLUSH;
    case PokerWinningType::Straight:
      return POKER_WINNING_CONFIDENCE_STRAIGHT;
    case PokerWinningType::ThreeOfAKind:
      return POKER_WINNING_CONFIDENCE_THREE_OF_A_KIND;
    case PokerWinningType::TwoPair:
      return POKER_WINNING_CONFIDENCE_TWO_PAIR;
    case PokerWinningType::Pair:
      return POKER_WINNING_CONFIDENCE_PAIR;
    default:
      return POKER_WINNING_CONFIDENCE_HIGH_CARD;
  }
}

struct Card PokerWinning::compare(
  const struct PokerWinning &left,
  const struct PokerWinning &right
) {
  uint8_t i;
  enum CardValue number = CardValue::Invalid;
  enum CardValue highNumberLeft = CardValue::Invalid;
  enum CardValue highNumberRight = CardValue::Invalid;
  struct Card card(0xFF), highCardLeft(0xFF), highCardRight(0xFF);
  int32_t index;
  uint8_t countCardsSame;

  countCardsSame = 0;

  for(i = 0; i < left.set.size(); i++) {
    card = left.set[i];
    number =  card.getValue();
    // Quick check
    if(highNumberLeft != CardValue::Invalid && number < highNumberLeft) continue;

    // Check if this number is within the other hand or not
    index = Card::containsValue(right.set, number);
    if(index != -1) {
      // This number IS within the other hand, let's check that the EXACT card
      // is a match/isn't a match.
      index = Card::contains(right.set, card);

      // Exact card match
      if(index != -1) {
        countCardsSame++;
        continue;
      }
      // Not exact card match.. ?
    }

    if(
      highNumberLeft == CardValue::Invalid ||
      number == CardValue::Ace ||
      highNumberLeft < number
    ) {
      highNumberLeft = number;
      highCardLeft = card;
    }
  }
  
  for(i = 0; i < right.set.size(); i++) {
    card = right.set[i];
    number = card.getValue();
    if(highNumberRight != CardValue::Invalid && number < highNumberRight) {
      continue;
    }
    
    index = Card::containsValue(left.set, number);
    if(index != -1) {
      index = Card::contains(left.set, card);
      if(index != -1) continue;
    }

    if(
      highNumberRight == CardValue::Invalid ||
      number == CardValue::Ace || highNumberRight < number
    ) {
      highNumberRight = number;
      highCardRight = card;
    }
  }


  if(countCardsSame == left.set.size()) {
    for(i = 0; i < left.set.size(); i++) {
      card = left.set[i];
      number = card.getValue();
      if(
        highNumberLeft == CardValue::Invalid ||
        number == CardValue::Ace ||
        highNumberLeft < number
      ) {
        highNumberLeft = number;
        highCardLeft = card;
      }
    }
    return highCardLeft;
  }

  if(highCardLeft.cardValue == 0xFF) return 0xFF;
  if(highNumberLeft < highNumberRight) return 0xFF;
  return highCardLeft;// Greater or Equal to.
}

void PokerWinning::fillRemaining() {
  uint8_t i;
  CardValue highest, current;
  struct Card highestCard(0x00);
  struct Card currentCard(0x00);

  // Set the kicker
  this->kicker = 0xFF;

  // Fill the remaining cards
  while(this->set.size() < POKER_WINNING_SET_SIZE) {
    highest = CardValue::Invalid;

    for(i = 0; i < this->full.size(); i++) {
      currentCard = this->full[i];
      if(Card::contains(this->set, currentCard) != -1) continue;

      if(highest == CardValue::Invalid) {
        highestCard = currentCard;
        highest = highestCard.getValue();
      } else {
        current = currentCard.getValue();
        if(current != CardValue::Ace && current < highest) continue;
        highestCard = currentCard;
        highest = current;
      }
    }

    if(highest == CardValue::Invalid) break;
    this->set.push_back(highestCard);
  }
  Card::sort(this->set);
}