// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "save/SaveFile.hpp"

namespace Dawn {
  class Game;
  
  enum class SaveResult {
    SUCCESS,
    FAILURE
  };
  
  enum class LoadResult {
    SUCCESS,
    FAILURE
  };

  class SaveManager final {
    private:
      std::weak_ptr<Game> game;
      struct SaveFile saveFile;
      int8_t currentSlot = -1;

    public:
      /**
       * Initializes the save manager and prepares it for use.
       * 
       * @param game The game to initialize the save manager for.
       */
      void init(std::shared_ptr<Game> game);

      /**
       * Immediately save the game to the given file. This will invoke the 
       * game's custom writer to write the output data to the save file.
       */
      enum SaveResult save();

      /**
       * Loads the current slotted save file. Invokes the internal managers read
       * function.
       */
      enum LoadResult load();
      
      /**
       * Deletes the given slot.
       * 
       * @param slot Slot to delete.
       */
      void deleteSlot(int8_t slot);

      /**
       * Returns the current slotted save file.
       * 
       * @returns The current slotted save file.
       */
      int8_t getCurrentSlot();

      /**
       * Sets the current slotted save file.
       * 
       * @param slot The slot to set.
       */
      void useSlot(int8_t slot);

      /**
       * Returns a list of used save slots, does not confirm if they are corrupt
       * or not, just whether they are in existance or not.
       * 
       * @return List of used save slots.
       */
      std::vector<int8_t> getSlots();
  };
}